/*
 *   ffilter.c
 *   metatemplate Vers. 0.10.0
 *                    Copyright (C)  2002-2010  Matteo Nastasi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *     CONTACT:    nastasi@alternativeoutput.it
 *                 matteo.nastasi@milug.org
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ffilter.h"

static FIFILE **zzh = NULL;
static int nzzh = 0;

FIFILE *ffopen(char *nome,char **keys)
{
  FIFILE *ff;
  int i;

  for (i = 0 ; i<nzzh ; i++) {
    if (zzh[i] == NULL)
      break;
  }
  if (i == nzzh) {
    nzzh++;
    if ((zzh = (FIFILE **)realloc(zzh,sizeof(FIFILE *)*nzzh)) == NULL)
      exit(100);
    zzh[i] = NULL;
  }
  if ((ff = (FIFILE *)malloc(sizeof(FIFILE))) == NULL)
    exit(101);
  if ((ff->bf[0] = (char *)malloc(FFDIMBUF+1)) == NULL) {
    free(ff);    
    exit(102);
  }
  /* PIPPO */
  // fprintf(stderr,"NOME %s",nome);
  if ((ff->fp[0] = fopen(nome,"r")) == NULL) {
    free(ff->bf[0]);
    free(ff);    
    exit(103);
  }
  ff->nc[0] = 0;
  ff->nfile = 1;
  ff->keys = keys;
  ff->index = i;
  ff->fkey = -1;
  ff->noctrlm = 1;
  // fprintf(stderr,"\nZZH [%d]\n",i);
  zzh[i] = ff;

  return (ff);
}

int ffinclude(FIFILE *ff,char *nome)
{


  if (ff == NULL || ff->nfile == FFNESTED)
    return (0);

  if ((ff->bf[ff->nfile] = (char *)malloc(FFDIMBUF+1)) == NULL)
    return(0);
  if ((ff->fp[ff->nfile] = fopen(nome,"r")) == NULL) {
    free(ff->bf[ff->nfile]);
    return(0);
  }
  ff->nc[ff->nfile] = 0;
  ff->nfile++;

  return (1);
}


int ffclose(FIFILE *ff) {
  int i;

  if (ff == NULL || zzh[ff->index] == NULL)
    return (0);
  
  zzh[ff->index] = NULL;
  for (i = 0 ; i < ff->nfile ; i++) {
    fclose(ff->fp[i]);
    free(ff->bf[i]);
  }
  free(ff);    

  return (1);
}

void ffchankeys(FIFILE *ff,char **keys)
{
  ff->keys = keys;
}

/*
 *  legge file finche' non vengono trovate chiavi
 */

int ffgetbin(char *bf,int size,FIFILE *ff)
{
  FILE *fp;
  char *p, *ffbf;
  int i, e, n, nf, minpos = FFDIMBUF,skip = 0,limin;

  /* indice del file corrente */
  nf = ff->nfile-1;
  /* file pointer del file corrente */
  fp = ff->fp[nf];
  /* buffer del file corrente */
  ffbf = ff->bf[nf];
  /* dimensione richiesta decurtata dello spazio per il \0 */
  size--;
  /* default della chiave trovata */
  ff->fkey = -1;

  /* se il numero di caratteri attualmente nel buffer e' attualmente minore
     della dim totale del buffer e il file non e' finito */
  while (ff->nc[nf] < FFDIMBUF && !feof(fp)) {
    /* pointer al primo char libero del buffer */
    p = &(ffbf[ff->nc[nf]]);
    /* tenta di riempire il buffer */
    n = fread(p,1,FFDIMBUF-ff->nc[nf],fp);

    /* se no control M cancella gli
       eventuali ^M letti ricompattando il buffer */
    if (ff->noctrlm) {
      for (i = 0 ; i < n ; i++) {
	if (p[i] == '\r') {
	  for (e = i ; e < n-1 ; e++)
	    p[e] = p[e+1];
	  n--;
	  i--;
	}
      }
    }  
    /* incrementa il contatore dei caratteri memorizzati nel buffer */
    ff->nc[nf] += n;
    /* aggiunge uno \0 in coda al buffer (per stampabilita')*/
    ffbf[ff->nc[nf]] = '\0';
  }
  /* limin viene settato al minimo tra il numero di caratteri letti e la
     meta della lunghezza del buffer */
  limin = (ff->nc[nf] < FFMEZBUF ? ff->nc[nf] : FFMEZBUF);
  /* se la dimensione richiesta e' maggiore di limin viene abbassata a tale
     valore */
  if (size > limin)
    size = limin;
  
  /* cerca una delle chiavi */
  for (i = 0 ; ff->keys[i] ; i++) {
    if ((p = strstr(ffbf,ff->keys[i])) != NULL) {
      if ((p - ffbf) < minpos) {
	minpos = p - ffbf;
	ff->fkey = i;
	skip = strlen(ff->keys[i]);
      }
    }
  }
  /* se la prima chiave trovata e' oltre in numero di car richiesti 
     non segnala il fatto che e' stata trovata */
  if (size < minpos) {
    ff->fkey = -1;
    skip = 0;
  }
  else 
    size = minpos;
  /* copia nel buffer i byte richiesti o cio' che precede la chiave */
  memcpy(bf,ffbf,size);
  bf[size] = '\0';
  /* trasla il buffer del numero di byte letti + event. lo skip dato
     dalla chiave */
  memmove(ffbf, &(ffbf[size+skip]),FFDIMBUF-(size+skip));
  /* allinea il numero di byte nel buffer */
  ff->nc[nf] -= (size+skip);
  
  /* se sonos stati letti 0 byte e non ci sono chiavi intercettate */
  if (size == 0 && ff->fkey == -1) {
    /* se il numero di file nello stack e' maggiore di 1 allora 
       chiude il file corrente e passa quello precedente altrimenti
       rende -1 per segnalare la fine della poss. di leggere dati */
    if (ff->nfile > 1) {
      ff->nfile--;
      fclose(ff->fp[ff->nfile]);
      free(ff->bf[ff->nfile]);
    }
    else
      size = -1;
  }
  return (size);
}

int ffkstat(FIFILE *ff) {
  return (ff->fkey);
}

char **ffgetkeys(FIFILE *ff) {
  return (ff->keys);
}

